 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

# Working Directory: project
# Input: (command line)
#   $1: filename of original json batch definition file
# Input: (file)
#   Risk-o/data/<$1>
#   Risk-o/data/previous_estimates.rds
#   Risk-o/cache/str_age_ns2003_now.rds
# Output: (file)
#   Risk-o/cache/V2<basename of $1>

library(jsonlite)
library(dplyr)

with(new.env(),{
  s <- commandArgs(trailingOnly = TRUE)
  #uncomment for testing
  #s <- 'Risk-o/data/stratum_25.json'
  p2 <- p <- jsonlite::read_json(path = s, simplifyVector = TRUE)
  Nhats <- readRDS('Risk-o/data/previous_estimates.rds')
  ns <- readRDS('Risk-o/cache/str_age_ns2003_now_selected.rds')
  (Nhats_slu <- Nhats %>% 
      filter(stratum == p$stratum) %>%
      select(-stratum, -Nh_EST)
  )
  p2$COMP_ID <- paste0('V2', p$COMP_ID)
  
  (n_slu <- ns %>% 
      filter(stratum_2003 == p$stratum, Agente_2003 == 'SLU') %>%
      summarize(n_2003 = sum(n * in_2003 ), n_now = sum(n))
  )
  n_min <- min(n_slu, Nhats_slu)
  n_max <- max(Nhats_slu, as.integer(n_slu$n_now * 1.8))
  p2$N_SLU <- as.integer(seq(from = n_min, to = n_max, length.out = 15))
  p2$N_EST <- rep(p$N_EST[1], 15)
  jsonlite::write_json(x = p2, path = paste0('Risk-o/cache/V2', basename(s)), pretty = TRUE)
})
