 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

#Calculates the "direct" estimate for Chungui excluding CEDAP database
# Working dir: project
#
# Input (files)
# 'Risk-o/src/fn_Risk-o.R'
# 'data/Data_2003.rds'
# Output (in memory)
# chungui_direct (numeric)

library(dplyr)
library(tidyr)
library(magrittr)
library(Rcapture)

with(new.env(), {
  source('Risk-o/src/fn_Risk-o.R', local = TRUE)
  (cvr <- readRDS('data/Data_2003.rds')  %>% 
     filter(estrato == 25) %>%
     rowwise() %>%
     mutate(ODH_NOCEDAP = as.numeric(any(COMISEDH==1, CNDDHH==1, CICR==1, DP_ONG==1))) %>%
     select(AGE_IF, CVR, DP, ODH, ODH_NOCEDAP) %>%
     ungroup()
  )
  #Without CEDAP
  (nocedap <- cvr %>%
      select(-ODH) %>%
      complete(AGE_IF, CVR, DP, ODH_NOCEDAP)%>%
      group_by_all() %>%
      tally() %>%
      rowwise() %>%
      filter(any(CVR==1, DP ==1, ODH_NOCEDAP == 1)) %>%
      ungroup() %>%
      filter(AGE_IF == 'SLU') %>%
      select(-AGE_IF)
  )
  (fit <- closedpMS.t(as.matrix(nocedap), dfreq = TRUE)$result %>%
      as.data.frame() %>%
      rownames_to_column('model') %>%
      filter(infoFit == 0) %>% #eliminate pathological models
      mutate(pval = 1 - pchisq(deviance, df = df)) %>% 
      arrange(desc(pval)) %>% 
      slice(1) %>% #since no pval > 0.05, we just take the largest 
      mutate(CI = paste0('[', paste0(round(abundance + 1.96 * c(-1, 1), digits = 0), collapse = ', '), ']')) %>%
      mutate(model = model %>% 
        {gsub('[\\[|\\]]', '',., perl = T)} %>% 
        {strsplit(., split = ',') %>% unlist %>% as.list()} %>% sapply(., FUN = function(x)strsplit(x, fixed=T, split = '')) %>% 
        {lapply(., function(x)paste0('[', paste(x, collapse = ','), ']'))} %>% unlist %>% 
        paste0(collapse = '')
      ) %>%
      mutate(model = gsub('1', 'CVR', model) %>% gsub('2', 'DP', .) %>% gsub('3', 'ODH', .))
    )
  fit$n2003 <- cvr %>% filter(AGE_IF == 'SLU') %>% NROW
  assign('chungui_direct', fit, envir = parent.env(environment()))
})
