 # 
 # Copyright (C) 2018 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

# support functions for simulation study
# OUTPUT: (in memory):various funcitons.
# Working Dir: project root
# Depends on: 
#   src/parameters.R
# 

library(tibble)
source('src/parameters.R')
cr_simu_params <- R6::R6Class(
  'basic_params', inherit = param_utils$CParams,
  public = list(
    initialize = function(){
      super$initialize(
        COMP_ID = 'TEST_COMP',
        stratum = 11,
        N_EST = 1552,
        N_SLU = 2380,
        smoothing_model = 'n ~ Agente*(CVR*ODH + DP*ODH + DP*CVR)',
        trials = 100
      )
    }
  )
)
full_table <- function(data_filtered, est_SLU, est_EST){
  data_filtered %>%
    select(Agente = AGE_IF, CVR = CVR, DP = DP, ODH = ODH) %>%
    filter(Agente %in% c('EST', 'SLU')) %>%
    mutate(n = 1) %>%
    complete(Agente, CVR, DP, ODH, fill = list(n=0))%>%
    group_by_at(vars(-n)) %>%
    summarize(n = sum(n)) %>%
    ungroup() %>%
    group_by(Agente) %>%
    mutate(N_g = ifelse(Agente == 'EST',est_EST, est_SLU)) %>%
    mutate(n = ifelse(CVR+DP+ODH == 0, max(1, N_g - sum(n)), n)) %>%
    ungroup() %>%
    select(-N_g)
}

fn_get_probs <- function(model = n~Agente*(CVR + DP + ODH + CVR*DP + CVR*ODH + DP *ODH) -1, data){
  m <- glm(formula = as.formula(model), data = data,  family = poisson)
  tab <- as.tibble(cbind(model.frame(m), e = m$fitted)) %>%
    mutate(p = e / sum(e)) %>%
    select(-e) %>%
    select(n, Agente, CVR, DP, ODH, p)
  return(tab)
}

fn_synth <- function(smooth){
  N <- sum(smooth$n)
  s <- smooth %>%
    select(-n) %>%
    mutate(n = rmultinom(1, size = N, p)) %>%
    filter(CVR + ODH + DP > 0) %>%
    select(-p)
  list(
    EST = s %>% filter(Agente == 'EST') %>% select(-Agente) %>% as.matrix,
    SLU = s %>% filter(Agente == 'SLU') %>% select(-Agente) %>% as.matrix,
    COMB = s %>% group_by(CVR, DP, ODH) %>% summarize(n = sum(n)) %>% as.matrix
  )
}

cr_direct <- function(x){
  a <- Rcapture::closedpMS.t(x, dfreq = TRUE, maxorder = 2)
  res <- a$results %>% as.tibble %>% mutate(p= 1 - pchisq(deviance, df = df), 
                                            q = ifelse(df == 0, Inf, deviance/df))
  f <- res %>% filter(p > 0.05, infoFit == 0) %>% arrange(q)
  Nhat <- NA
  if(NROW(f) > 0){
    Nhat <- slice(f,1)$abundance
  }
  list(fit = res, Nhat=Nhat)
}
